import cherrypy # externí knihovna pro práci s webovým serverem
from pathlib import Path    # knihovna pro práci s cestami (soubory, adresáře)
from PIL import Image, ImageOps     # externí knihovna pro práci s obrázky
 

# náš webový server pro aplikaci zpracování obrázků
class WebServer:
    
    # init serveru: pokud neexistuje adresář pro ukládání nahraných obrázků (a výsledků jeho zpracování), vytvoříme ho
    def __init__(self):
        soubory = Path('public')
        if not soubory.exists():
            soubory.mkdir()
    
    # hlavní metoda zobrazující vstupní stránku aplikace
    @cherrypy.expose
    def index(self):
        return f"""
            Zdraví Vás Váš osobní webový server!
            <br>
            <form method="post" action="upload" enctype="multipart/form-data">
                <input type="file" name="soubor">
                <input type="submit">
            </form>
        """
    
    # metoda reagující na nahrání obrázku na adresu 'http://localhost:8080/upload' (kam pošle obrázek formulář z metody 'index()')
    @cherrypy.expose
    def upload(self, soubor):
        # načtení a uložení dat nahraného obrázku
        # ~ můžete klidně pracovat jen s daty v paměti, ale na disku se na ty obrázky líp kouká a kontroluje se to
        data = soubor.file.read()
        with open(f'public/{soubor.filename}', 'bw') as f:
            f.write(data)
        # výpočet a uložení negativu (pomocí knihovny Pillow)
        obrázek = Image.open(f'public/{soubor.filename}')
        negativ = ImageOps.invert(obrázek)
        negativ.save('public/negativ.jpg')
        # zobrazení výsledné HTML-stránky
        return f"""
            binární data o délce {len(data)} bajtů
            <br>
            <img src="soubory/{soubor.filename}">
            <br>
            <img src="soubory/negativ.jpg">
        """


# spuštění HTTP-serveru (tradičně takto oklikou přes '__name__')
if __name__ == '__main__':
    config = {
        '/soubory': {   # název adresáře 'public' pro použití uvnitř vraceného HTML-textu
            'tools.staticdir.on': True,
            'tools.staticdir.root': Path().cwd().resolve(), # absolutní cesta k místu spuštění serveru
            'tools.staticdir.dir': './public',  # skutečné jméno adresáře pro ukládání souborů relativně k místu spuštění serveru
        },
    }
    cherrypy.quickstart(WebServer(), '/', config)
